// Java 3Dƥȥץ
// SimpleKeyBehavior.java
//   Copyright (c) 1999 ENDO Yasuyuki
//                      mailto:yasuyuki@javaopen.org
//                      http://www.javaopen.org/j3dbook/index.html

import java.awt.event.*;
import java.awt.AWTEvent;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class SimpleKeyBehavior extends Behavior {
  private static final double ANGLE = Math.PI / 180.0;
  private static final double STEP = 0.1;
  private Transform3D t3d = new Transform3D();
  private Transform3D ct3d = new Transform3D();
  private Matrix4d matrix = new Matrix4d();
  private TransformGroup trans = null;
  private WakeupOnAWTEvent kpress = null;

  public SimpleKeyBehavior(TransformGroup aTrans) {
    trans = aTrans;
  }

  public void initialize() {
    kpress = new WakeupOnAWTEvent(KeyEvent.KEY_PRESSED);
    wakeupOn(kpress);
  }

  public void processStimulus(Enumeration criteria) {
    WakeupOnAWTEvent wevent = null;
    AWTEvent[] events = null;
    while (criteria.hasMoreElements()) {
      wevent = (WakeupOnAWTEvent)criteria.nextElement();
      events = wevent.getAWTEvent();
      for (int i=0; i<events.length; i++) {
	if (events[i] instanceof KeyEvent) {
	  int code = ((KeyEvent)events[i]).getKeyCode();
	  switch (code) {
	  case KeyEvent.VK_UP:
	    //System.out.println("VK_UP");//DEBUG
	    t3d.set(new Vector3d(0.0, 0.0, -STEP));
	    trans.getTransform(ct3d);
	    ct3d.mul(t3d);
	    trans.setTransform(ct3d);
	    break;
	  case KeyEvent.VK_DOWN:
	    //System.out.println("VK_DOWN");//DEBUG
	    t3d.set(new Vector3d(0.0, 0.0, STEP));
	    trans.getTransform(ct3d);
	    ct3d.mul(t3d);
	    trans.setTransform(ct3d);
	    break;
	  case KeyEvent.VK_RIGHT:
	    //System.out.println("VK_RIGHT");//DEBUG
	    t3d.rotY(-ANGLE);
	    trans.getTransform(ct3d);
	    ct3d.get(matrix);
	    ct3d.setTranslation(new Vector3d(0.0, 0.0, 0.0));
	    ct3d.mul(t3d);
	    ct3d.setTranslation(new Vector3d(matrix.m03, matrix.m13, matrix.m23));
	    trans.setTransform(ct3d);
	    break;
	  case KeyEvent.VK_LEFT:
	    //System.out.println("VK_LEFT");//DEBUG
	    t3d.rotY(ANGLE);
	    trans.getTransform(ct3d);
	    ct3d.get(matrix);
	    ct3d.setTranslation(new Vector3d(0.0, 0.0, 0.0));
	    ct3d.mul(t3d);
	    ct3d.setTranslation(new Vector3d(matrix.m03, matrix.m13, matrix.m23));
	    trans.setTransform(ct3d);
	    break;
	  }
	}
      }
    }
    wakeupOn(kpress);
  }

  public Node cloneNodeComponent(boolean forceDuplication) {
     SimpleKeyBehavior simpleKeyBehavior = new SimpleKeyBehavior(trans);
     simpleKeyBehavior.duplicateNode(this, forceDuplication);
     return simpleKeyBehavior;
  }

  public void duplicateNode(Node node, boolean forceDuplication) {
    super.duplicateNode(node, forceDuplication);
  }

  public void updateNodeReference(NodeReferenceTable table) {
    super.updateNodeReferences(table);
    TransformGroup newTransformGroup = 
      (TransformGroup)table.getNewObjectReference(trans);
    trans = newTransformGroup;
  }
}

