// Java 3D Test Applet
// EarthApplet.java
//   Copyright (c) 1999 ENDO Yasuyuki
//                      mailto:yasuyuki@javaopen.org
//                      http://www.javaopen.org/j3dbook/index.html

import java.applet.*;
import java.awt.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.image.TextureLoader;
import com.sun.j3d.utils.behaviors.picking.PickRotateBehavior;
import com.sun.j3d.utils.behaviors.picking.PickTranslateBehavior;
import com.sun.j3d.utils.behaviors.picking.PickZoomBehavior;

public class EarthApplet extends Applet {
  private Canvas3D canvas = null;

  private boolean isStandalone = false;

  public EarthApplet() {
    this(false);
  }
  
  public EarthApplet(boolean isStandalone) {
    this.isStandalone = isStandalone;
  }

  public void init() {
    GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();
    this.canvas = new Canvas3D(config);
    this.setLayout(new BorderLayout());
    this.add(this.canvas, BorderLayout.CENTER);

    SimpleUniverse universe = new SimpleUniverse(canvas);
    universe.getViewingPlatform().setNominalViewingTransform();
  
    universe.addBranchGraph(createSceneGraph());
  }

  private BranchGroup createSceneGraph() {
    BranchGroup root = new BranchGroup();
    
    TransformGroup trans = new TransformGroup();
    trans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    trans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    trans.setCapability(TransformGroup.ENABLE_PICK_REPORTING);

    BoundingSphere bounds = new BoundingSphere(new Point3d(), 100.0);
    
    PickRotateBehavior rotator =
      new PickRotateBehavior(root, this.canvas, bounds);
    root.addChild(rotator);
    
    PickTranslateBehavior translator =
      new PickTranslateBehavior(root, this.canvas, bounds);
    root.addChild(translator);
    
    PickZoomBehavior zoomer =
      new PickZoomBehavior(root, this.canvas, bounds);
    root.addChild(zoomer);
    
    Sphere earth = new Sphere( 0.4f,
                               Sphere.GENERATE_TEXTURE_COORDS,
                               createAppearance());
    trans.addChild(earth);
    
    root.addChild(trans);
    
    return root;
  }

  private Appearance createAppearance() {
    Appearance app = new Appearance();

    Image image = null;
    if (this.isStandalone) {
      Toolkit toolkit = Toolkit.getDefaultToolkit();
      image = toolkit.getImage("earth.jpg");
    } else {
      image = getImage(getCodeBase(), "earth.jpg");
    }
    MediaTracker mt = new MediaTracker(this);
    mt.addImage(image, 0);
    mt.checkAll(true);
    try { mt.waitForID(0); } catch (InterruptedException e) { e.printStackTrace(); }

    Texture texture = new TextureLoader(image, this).getTexture();
    
    app.setTexture(texture);
    
    return app;
  }

  public static void main(String[] args) {
    EarthApplet applet = new EarthApplet(true); // isStandalone = true;
    Frame frame = new MainFrame(applet, 500, 500);
  }
}
