// Java 3D Test Applet
// MouseBehaviorTest.java
//   Copyright (c) 1999 ENDO Yasuyuki
//                      mailto:yasuyuki@javaopen.org
//                      http://www.javaopen.org/j3dbook/index.html

import java.applet.*;
import java.awt.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.image.TextureLoader;
import com.sun.j3d.utils.behaviors.mouse.MouseBehavior;
import com.sun.j3d.utils.behaviors.mouse.MouseRotate;
import com.sun.j3d.utils.behaviors.mouse.MouseTranslate;
import com.sun.j3d.utils.behaviors.mouse.MouseZoom;

public class MouseBehaviorTest extends Applet {
  private SimpleUniverse universe = null;

  private boolean isStandalone = false;

  public MouseBehaviorTest() {
    this(false);
  }
  
  public MouseBehaviorTest(boolean isStandalone) {
    this.isStandalone = isStandalone;
  }

  public void init() {
    GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();
    Canvas3D canvas = new Canvas3D(config);
    this.setLayout(new BorderLayout());
    this.add(canvas, BorderLayout.CENTER);

    universe = new SimpleUniverse(canvas);
    universe.getViewingPlatform().setNominalViewingTransform();
  
    universe.addBranchGraph(createSceneGraph());
  }

  private BranchGroup createSceneGraph() {
    BranchGroup root = new BranchGroup();
    
    TransformGroup trans = new TransformGroup();

    TransformGroup viewtrans = universe.getViewingPlatform().getViewPlatformTransform();

    BoundingSphere bounds = new BoundingSphere(new Point3d(), 100.0);
    
    MouseRotate rotator = new MouseRotate(viewtrans);
    //MouseRotate rotator = new MouseRotate(MouseBehavior.INVERT_INPUT);
    //rotator.setTransformGroup(viewtrans);
    rotator.setSchedulingBounds(bounds);
    root.addChild(rotator);
    
    MouseTranslate translator = new MouseTranslate(viewtrans);
    //MouseTranslate translator = new MouseTranslate(MouseBehavior.INVERT_INPUT);
    //translator.setTransformGroup(viewtrans);
    translator.setSchedulingBounds(bounds);
    root.addChild(translator);
    
    MouseZoom zoomer = new MouseZoom(viewtrans);
    //MouseZoom zoomer = new MouseZoom(MouseBehavior.INVERT_INPUT);
    //zoomer.setTransformGroup(viewtrans);
    zoomer.setSchedulingBounds(bounds);
    root.addChild(zoomer);
    
    Sphere earth = new Sphere( 0.4f,
                               Sphere.GENERATE_TEXTURE_COORDS,
                               createAppearance());
    trans.addChild(earth);
    
    root.addChild(trans);
    
    return root;
  }

  private Appearance createAppearance() {
    Appearance app = new Appearance();

    // Texture $B$N@8@.(B
    Image image = null;
    if (this.isStandalone) {
      // $B%"%W%j%1!<%7%g%s$H$7$F<B9T$5$l$F$$$k(B
      Toolkit toolkit = Toolkit.getDefaultToolkit();
      image = toolkit.getImage("earth.jpg");
    } else {
      // $B%"%W%l%C%H$H$7$F<B9T$5$l$F$$$k(B
      image = getImage(getCodeBase(), "earth.jpg");
    }
    MediaTracker mt = new MediaTracker(this);
    mt.addImage(image, 0);
    mt.checkAll(true);
    try { mt.waitForID(0); } catch (InterruptedException e) { e.printStackTrace(); }
    Texture texture = new TextureLoader(image, this).getTexture();
    
    app.setTexture(texture);
    
    return app;
  }

  public static void main(String[] args) {
    MouseBehaviorTest applet = new MouseBehaviorTest(true); // isStandalone = true;
    Frame frame = new MainFrame(applet, 500, 500);
  }
}
