// Java 3D Test Applet
// PickMouseTest.java
//   Copyright (c) 1999-2001 ENDO Yasuyuki
//                      mailto:yasuyuki@javaopen.org
//                      http://www.javaopen.org/j3dbook/index.html

import java.applet.*;
import java.awt.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.geometry.Primitive;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.geometry.Cylinder;
import com.sun.j3d.utils.geometry.Cone;
import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.image.TextureLoader;
import com.sun.j3d.utils.picking.PickTool;
import com.sun.j3d.utils.picking.PickResult;
import com.sun.j3d.utils.picking.behaviors.PickRotateBehavior;
import com.sun.j3d.utils.picking.behaviors.PickTranslateBehavior;
import com.sun.j3d.utils.picking.behaviors.PickZoomBehavior;

public class PickMouseTest extends Applet {
  private Canvas3D canvas = null;

  private boolean isStandalone = false;

  public PickMouseTest() {
    this(false);
  }
  
  public PickMouseTest(boolean isStandalone) {
    this.isStandalone = isStandalone;
  }

  public void init() {
    GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();
    canvas = new Canvas3D(config);
    this.setLayout(new BorderLayout());
    this.add(canvas, BorderLayout.CENTER);

    SimpleUniverse universe = new SimpleUniverse(canvas);
    universe.getViewingPlatform().setNominalViewingTransform();
  
    BranchGroup scene = createSceneGraph();
  
    universe.addBranchGraph(scene);
  }

  private BranchGroup createSceneGraph() {
    BranchGroup root = new BranchGroup();
    
    BoundingSphere bounds = new BoundingSphere( new Point3d(), 100.0 );
    
    PickRotateBehavior rotator =
      new PickRotateBehavior(root, canvas, bounds);
    rotator.setMode(PickTool.GEOMETRY);
    root.addChild(rotator);
    
    PickTranslateBehavior translator =
      new PickTranslateBehavior(root, canvas, bounds);
    translator.setMode(PickTool.GEOMETRY);
    root.addChild(translator);
    
    PickZoomBehavior zoomer =
      new PickZoomBehavior(root, canvas, bounds);
    zoomer.setMode(PickTool.GEOMETRY);
    root.addChild(zoomer);
    
    SimplePicking picker =
      new SimplePicking( root, canvas, bounds,
			 PickTool.GEOMETRY_INTERSECT_INFO, PickResult.PRIMITIVE );
    picker.setupCallback( new SimplePickingCallback() {
      public void picked(int type, Node node) {
	if (node != null) {
	  String data = (String)node.getUserData();
	  System.out.println(data);
	} else {
	  System.out.println("Error: node is null.");
	}
      }
    });
    root.addChild(picker);
    
    TransformGroup trans = new TransformGroup();
    root.addChild(trans);
    
    TransformGroup gtrans = new TransformGroup();
    gtrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    gtrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    gtrans.setCapability(TransformGroup.ENABLE_PICK_REPORTING);
    trans.addChild(gtrans);

    double[] vertices = { -0.8, -0.8, 0.0,
                          -0.8,  0.8, 0.0,
                          -0.4, -0.8, 0.0,
                          -0.4,  0.8, 0.0,
                           0.0, -0.8, 0.0,
                           0.0,  0.8, 0.0,
                           0.4, -0.8, 0.0,
                           0.4,  0.8, 0.0,
                           0.8, -0.8, 0.0,
                           0.8,  0.8, 0.0,
                          -0.8, -0.8, 0.0,
                           0.8, -0.8, 0.0,
                          -0.8, -0.4, 0.0,
                           0.8, -0.4, 0.0,
                          -0.8,  0.0, 0.0,
                           0.8,  0.0, 0.0,
                          -0.8,  0.4, 0.0,
                           0.8,  0.4, 0.0,
                          -0.8,  0.8, 0.0,
                           0.8,  0.8, 0.0 };

    LineArray geom = new LineArray( vertices.length, GeometryArray.COORDINATES);
    geom.setCoordinates(0, vertices);

    Shape3D grid = new Shape3D(geom);
    grid.setPickable(false); // sbLȎΏۂ珜O
    gtrans.addChild(grid);

    Appearance ap = createAppearance();

    Transform3D bt3d = new Transform3D();
    bt3d.set(new Vector3d(-0.4, 0.4, 0.0));
    TransformGroup btrans = new TransformGroup(bt3d);
    btrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    btrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    btrans.setCapability(TransformGroup.ENABLE_PICK_REPORTING);
    trans.addChild(btrans);

    Box box = 
      new Box( 0.25f, 0.25f, 0.25f,
               Primitive.GENERATE_TEXTURE_COORDS |
	       Primitive.ENABLE_GEOMETRY_PICKING,
               ap );
    box.setUserData("this is box.");

    //box.getShape(Box.TOP).setCapability(Shape3D.ALLOW_GEOMETRY_READ);//DEBUG
    //box.getShape(Box.BOTTOM).setCapability(Shape3D.ALLOW_GEOMETRY_READ);//DEBUG
    //box.getShape(Box.FRONT).setCapability(Shape3D.ALLOW_GEOMETRY_READ);//DEBUG
    //box.getShape(Box.BACK).setCapability(Shape3D.ALLOW_GEOMETRY_READ);//DEBUG
    //box.getShape(Box.LEFT).setCapability(Shape3D.ALLOW_GEOMETRY_READ);//DEBUG
    //box.getShape(Box.RIGHT).setCapability(Shape3D.ALLOW_GEOMETRY_READ);//DEBUG

    //((GeometryArray)box.getShape(Box.TOP).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.BOTTOM).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.FRONT).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.BACK).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.LEFT).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.RIGHT).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);//DEBUG

    //((GeometryArray)box.getShape(Box.TOP).getGeometry()).setCapability(GeometryArray.ALLOW_FORMAT_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.BOTTOM).getGeometry()).setCapability(GeometryArray.ALLOW_FORMAT_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.FRONT).getGeometry()).setCapability(GeometryArray.ALLOW_FORMAT_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.BACK).getGeometry()).setCapability(GeometryArray.ALLOW_FORMAT_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.LEFT).getGeometry()).setCapability(GeometryArray.ALLOW_FORMAT_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.RIGHT).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);//DEBUG

    //((GeometryArray)box.getShape(Box.TOP).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.BOTTOM).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.FRONT).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.BACK).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.LEFT).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);//DEBUG
    //((GeometryArray)box.getShape(Box.RIGHT).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);//DEBUG

    btrans.addChild(box);
    
    Transform3D cyt3d = new Transform3D();
    cyt3d.set(new Vector3d(-0.4, -0.4, 0.0));
    TransformGroup cytrans = new TransformGroup(cyt3d);
    cytrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    cytrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    cytrans.setCapability(TransformGroup.ENABLE_PICK_REPORTING);
    trans.addChild(cytrans);

    Cylinder cylinder =
      new Cylinder( 0.3f, 0.6f,
		    Primitive.GENERATE_TEXTURE_COORDS |
               	    Primitive.ENABLE_GEOMETRY_PICKING,
		    ap );
    cylinder.setUserData("this is cylinder.");

    //cylinder.getShape(Cylinder.TOP).setCapability(Shape3D.ALLOW_GEOMETRY_READ);//DEBUG
    //cylinder.getShape(Cylinder.BOTTOM).setCapability(Shape3D.ALLOW_GEOMETRY_READ);//DEBUG
    //cylinder.getShape(Cylinder.BODY).setCapability(Shape3D.ALLOW_GEOMETRY_READ);//DEBUG

    //((GeometryArray)cylinder.getShape(Cylinder.TOP).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);//DEBUG
    //((GeometryArray)cylinder.getShape(Cylinder.BOTTOM).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);//DEBUG
    //((GeometryArray)cylinder.getShape(Cylinder.BODY).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);//DEBUG

    //((GeometryArray)cylinder.getShape(Cylinder.TOP).getGeometry()).setCapability(GeometryArray.ALLOW_FORMAT_READ);//DEBUG
    //((GeometryArray)cylinder.getShape(Cylinder.BOTTOM).getGeometry()).setCapability(GeometryArray.ALLOW_FORMAT_READ);//DEBUG
    //((GeometryArray)cylinder.getShape(Cylinder.BODY).getGeometry()).setCapability(GeometryArray.ALLOW_FORMAT_READ);//DEBUG

    //((GeometryArray)cylinder.getShape(Cylinder.TOP).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);//DEBUG
    //((GeometryArray)cylinder.getShape(Cylinder.BOTTOM).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);//DEBUG
    //((GeometryArray)cylinder.getShape(Cylinder.BODY).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);//DEBUG

    cytrans.addChild(cylinder);
    
    Transform3D cnt3d = new Transform3D();
    cnt3d.set(new Vector3d(0.4, 0.4, 0.0));
    TransformGroup cntrans = new TransformGroup(cnt3d);
    cntrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    cntrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    cntrans.setCapability(TransformGroup.ENABLE_PICK_REPORTING);
    trans.addChild(cntrans);
    
    Cone cone = new Cone( 0.3f, 0.6f,
                          Primitive.GENERATE_TEXTURE_COORDS |
                          Primitive.ENABLE_GEOMETRY_PICKING,
			  ap);
    cone.setUserData("this is cone.");

    //cone.getShape(Cone.CAP).setCapability(Shape3D.ALLOW_GEOMETRY_READ);//DEBUG
    //cone.getShape(Cone.BODY).setCapability(Shape3D.ALLOW_GEOMETRY_READ);//DEBUG

    //((GeometryArray)cone.getShape(Cone.CAP).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);//DEBUG
    //((GeometryArray)cone.getShape(Cone.BODY).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);//DEBUG

    //((GeometryArray)cone.getShape(Cone.CAP).getGeometry()).setCapability(GeometryArray.ALLOW_FORMAT_READ);//DEBUG
    //((GeometryArray)cone.getShape(Cone.BODY).getGeometry()).setCapability(GeometryArray.ALLOW_FORMAT_READ);//DEBUG

    //((GeometryArray)cone.getShape(Cone.CAP).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);//DEBUG
    //((GeometryArray)cone.getShape(Cone.BODY).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);//DEBUG

    cntrans.addChild(cone);

    Transform3D st3d = new Transform3D();
    st3d.set(new Vector3d(0.4, -0.4, 0.0));
    TransformGroup strans = new TransformGroup(st3d);
    strans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    strans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    strans.setCapability(TransformGroup.ENABLE_PICK_REPORTING);
    trans.addChild(strans);
    
    Sphere sphere = new Sphere( 0.3f,
                                Primitive.GENERATE_TEXTURE_COORDS |
                                Primitive.ENABLE_GEOMETRY_PICKING,
                                ap );
    sphere.setUserData("this is sphere.");

    //sphere.getShape(Sphere.BODY).setCapability(Shape3D.ALLOW_GEOMETRY_READ);//DEBUG
    //((GeometryArray)sphere.getShape(Sphere.BODY).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);//DEBUG
    //((GeometryArray)sphere.getShape(Sphere.BODY).getGeometry()).setCapability(GeometryArray.ALLOW_FORMAT_READ);//DEBUG
    //((GeometryArray)sphere.getShape(Sphere.BODY).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);//DEBUG

    strans.addChild(sphere);
        
    return root;
  }
  
  private Appearance createAppearance() {
    Appearance app = new Appearance();

    // Texture ̐
    Image image = null;
    if (this.isStandalone) {
      // AvP[VƂĎsĂ
      Toolkit toolkit = Toolkit.getDefaultToolkit();
      image = toolkit.getImage("earth.jpg");
    } else {
      // AvbgƂĎsĂ
      image = getImage(getCodeBase(), "earth.jpg");
    }
    MediaTracker mt = new MediaTracker(this);
    mt.addImage(image, 0);
    mt.checkAll(true);
    try { mt.waitForID(0); } catch (InterruptedException e) { e.printStackTrace(); }
    Texture texture = new TextureLoader(image, this).getTexture();
    
    app.setTexture(texture);
    
    return app;
  }

  public static void main(String[] args) {
    PickMouseTest applet = new PickMouseTest(true); // isStandalone = true;
    Frame frame = new MainFrame(applet, 500, 500);
  }
}
