// Java 3D Test Program
// MultiViewTest.java
// Copyright(c) 1999 ENDO Yasuyuki <yasuyuki@javaopen.org>

import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.ColorCube;
import com.sun.j3d.utils.behaviors.mouse.*;
import java.awt.*;

public class MultiViewTest {
  public MultiViewTest() {

    SmallUniverse u1 = new SmallUniverse();
    Locale locale = u1.getLocale();
    SmallUniverse u2 = new SmallUniverse(locale);
    u2.getView().setDepthBufferFreezeTransparent(true);
    
    BranchGroup scene = new BranchGroup();

    BoundingSphere bounds = new BoundingSphere(new Point3d(), 100.0);

    Background bg = new Background(new Color3f(1.0f, 1.0f, 1.0f));
    bg.setApplicationBounds(bounds);
    scene.addChild(bg);

    scene.addChild( new ColorCube(0.4) );

    double[] averts = { -5.0,  0.0,  0.0,
			 5.0,  0.0,  0.0,
			 0.0, -5.0,  0.0,
			 0.0,  5.0,  0.0,
			 0.0,  0.0, -5.0,
			 0.0,  0.0,  5.0 };
    float[] acolors = { 1.0f, 0.0f, 0.0f,
			1.0f, 0.0f, 0.0f,
			0.0f, 1.0f, 0.0f,
			0.0f, 1.0f, 0.0f,
			0.0f, 0.0f, 1.0f,
			0.0f, 0.0f, 1.0f };
    LineArray aarray =
      new LineArray( averts.length/3,
		     GeometryArray.COORDINATES |
		     GeometryArray.COLOR_3 );
    aarray.setCoordinates(0, averts);
    aarray.setColors(0, acolors);
    Shape3D axes = new Shape3D(aarray);
    scene.addChild(axes);

    Transform3D t3d1 = new Transform3D();
    t3d1.set( new Vector3d(0.0, 0.0, 2.41) );
    u1.setTransform(t3d1);

    u1.detachBranchGraph();

    double[] pverts = { 0.0, 0.0, 0.0 };
    float[] pcolors = { 0.0f, 0.0f, 1.0f };
    PointArray parray = new PointArray( pverts.length/3,
					GeometryArray.COORDINATES | GeometryArray.COLOR_3);
    parray.setCoordinates(0, pverts);
    parray.setColors(0, pcolors);
    Appearance papp = new Appearance();
    PointAttributes pattr = new PointAttributes();
    pattr.setPointSize(4.0f);
    papp.setPointAttributes(pattr);
    Shape3D points = new Shape3D(parray, papp);
    u1.getTransformGroup().addChild(points);

    double tan =  Math.tan(u1.getView().getFieldOfView() / 2.0);
    double fclip = u1.getView().getFrontClipDistance();
    double fp = fclip * tan;
    double bclip = u1.getView().getBackClipDistance();
    double bp = bclip * tan;
    System.out.println("tan=" + tan + ", fclip=" + fclip + ", bclip=" + bclip);
    double[] lverts = {  0.0, 0.0, 0.0,
                        -fp, -fp, -fclip,
			 fp, -fp, -fclip,
			 fp,  fp, -fclip,
                        -fp,  fp, -fclip, 
                        -bp, -bp, -bclip, 
                         bp, -bp, -bclip, 
                         bp,  bp, -bclip, 
                        -bp,  bp, -bclip };
    int[] lindices = { 1, 2,
		       2, 3,
		       3, 4,
		       4, 1,
		       5, 6,
		       6, 7,
		       7, 8,
		       8, 5,
		       1, 5,
		       2, 6,
		       3, 7,
		       4, 8 };
    IndexedLineArray larray = new IndexedLineArray( lverts.length/3,
						    GeometryArray.COORDINATES,
						    lindices.length );
    larray.setCoordinates(0, lverts);
    larray.setCoordinateIndices(0, lindices);
    Appearance lapp = new Appearance();
    ColoringAttributes cattr =
      new ColoringAttributes( new Color3f(0.0f, 0.0f, 0.0f), ColoringAttributes.SHADE_FLAT );
    lapp.setColoringAttributes(cattr);
    Shape3D lines = new Shape3D(larray, lapp);
    u1.getTransformGroup().addChild(lines);
    
    double iplate = 0.4572;
    double ip = iplate * tan;
    double[] qverts = { -ip, -ip, -iplate,
                         ip, -ip, -iplate,
			 ip,  ip, -iplate,
			-ip,  ip, -iplate };
    float[] qcolors = { 1.0f, 0.0f, 1.0f, 0.3f,
			1.0f, 0.0f, 1.0f, 0.3f,
			1.0f, 0.0f, 1.0f, 0.3f,
			1.0f, 0.0f, 1.0f, 0.3f };
    QuadArray qarray =
      new QuadArray(qverts.length/3, GeometryArray.COORDINATES | GeometryArray.COLOR_4);
    qarray.setCoordinates(0, qverts);
    qarray.setColors(0, qcolors);
    Shape3D imageplate = new Shape3D(qarray);
    u1.getTransformGroup().addChild(imageplate);
    
    u1.atachBranchGraph();
    
    Transform3D t3d2 = new Transform3D();
    Vector3d up = new Vector3d(-0.57, 0.57, -0.57);
    //Vector3d up = new Vector3d(0.0, 1.0, 0.0);
    up.normalize();
    t3d2.lookAt( new Point3d(12.0, 5.0, 8.0),      // eye
                 new Point3d(0.0, 0.0, 0.0),        // center
                 up ); // up
    t3d2.invert(); // K{
    t3d2.normalize();
    u2.setTransform(t3d2);

    MouseRotate rot = new MouseRotate(u2.getTransformGroup());
    rot.setSchedulingBounds(bounds);
    scene.addChild(rot);
 
    MouseTranslate tra = new MouseTranslate(u2.getTransformGroup());
    tra.setSchedulingBounds(bounds);
    scene.addChild(tra);
 
    MouseZoom zom = new MouseZoom(u2.getTransformGroup());
    zom.setSchedulingBounds(bounds);
    scene.addChild(zom);
 
   locale.addBranchGraph(scene);
   
    Frame f1 = new Frame();
    f1.setLayout( new BorderLayout() );
    f1.add(u1.getCanvas(), BorderLayout.CENTER);
    f1.pack();
    f1.setSize(200, 200);
    f1.show();
    
    Frame f2 = new Frame();
    f2.setLayout( new BorderLayout() );
    f2.add(u2.getCanvas(), BorderLayout.CENTER);
    f2.pack();
    f2.setSize(200, 200);
    f2.show();
  }
  
  public static void main(String[] args) {
    MultiViewTest MultiViewTest = new MultiViewTest();
  }
}

