// Java 3D Test Program
// SmallUniverse.java
// Copyright(c) 1999 ENDO Yasuyuki <yasuyuki@javaopen.org>

import javax.media.j3d.*;

public class SmallUniverse {
  protected VirtualUniverse universe = null;
  protected Locale locale = null;
  protected BranchGroup root = null;
  protected TransformGroup trans = null;
  protected ViewPlatform vp = null;
  protected View view = null;
  protected PhysicalBody body = null;
  protected PhysicalEnvironment env = null;
  protected Canvas3D canvas = null;
  
  public SmallUniverse() {
    this( new Locale( new VirtualUniverse() ) );
  }
  
  public SmallUniverse(Locale locale) {
    universe = locale.getVirtualUniverse();

    this.locale = locale;

    root = new BranchGroup();
    root.setCapability(BranchGroup.ALLOW_DETACH);
    root.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);

    trans = new TransformGroup();
    trans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    trans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    root.addChild(trans);

    vp = new ViewPlatform();
    float r = vp.getActivationRadius();//DEBUG
    System.out.println("r=" + r);//DEBUG
    trans.addChild(vp);

    view = new View();
    view.attachViewPlatform(vp);

    body = new PhysicalBody();
    view.setPhysicalBody(body);

    env = new PhysicalEnvironment();
    view.setPhysicalEnvironment(env);
    
    canvas = new Canvas3D(null);
    view.addCanvas3D(canvas);

    this.locale.addBranchGraph(root);
  }
  
  public VirtualUniverse getUniverse() { return universe; }

  public Locale getLocale() { return locale; }
  //public void setLocale(Locale locale) { this.locale = locale; }

  public BranchGroup getBranchGroup() { return root; }

  public TransformGroup getTransformGroup() { return trans; }

  public void getTransform(Transform3D t3d) { trans.getTransform(t3d); }
  public void setTransform(Transform3D t3d) { trans.setTransform(t3d); }
  
  public ViewPlatform getViewPlatform() { return vp; }

  public View getView() { return view; }

  public PhysicalBody getPhysicalBody() { return body; }
  
  public PhysicalEnvironment getPhysicalEnvironment() { return env; }

  public Canvas3D getCanvas() { return canvas; }
  //public void setCanvas3D(Canvas3D canvas) { this.canvas = canvas; }

  public void detachBranchGraph() {
    java.util.Enumeration e = locale.getAllBranchGraphs();
    while (e.hasMoreElements()) {
      BranchGroup g = (BranchGroup)e.nextElement();
      if (g == root) locale.removeBranchGraph(root);
    }
  }
  
  public void atachBranchGraph() {
    java.util.Enumeration e = locale.getAllBranchGraphs();
    while (e.hasMoreElements()) {
      BranchGroup g = (BranchGroup)e.nextElement();
      if (g == root) return;
    }
    locale.addBranchGraph(root);
  }
  
}
