// Java 3D Test Applet
// Text2DTest.java
//   Copyright (c) 1999 ENDO Yasuyuki
//                      mailto:yasuyuki@javaopen.org
//                      http://www.javaopen.org/j3dbook/index.html

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.geometry.Text2D;
import com.sun.j3d.utils.image.TextureLoader;

public class Text2DTest extends Applet {
  boolean isStandalone = false;
  
  private Canvas3D canvas = null;
  private SimpleUniverse universe = null;
  private BranchGroup scene = null;
  private Background background = null;
  private TextureLoader loader = null;

  private Color3f bgcolor = new Color3f(0.0f, 0.0f, 0.0f);   // wiF 
  private Color3f fontcolor = new Color3f(1.0f, 0.0f, 0.0f); // tHgF 
  private Vector3f fontposition = new Vector3f(0.0f, 0.0f, 0.0f); // tHgʒu

  private TuplePanel fcpanel = new TuplePanel(fontcolor);
  private TuplePanel fppanel = new TuplePanel(fontposition);

  public Text2DTest() {
    this(false);
  }
  
  public Text2DTest(boolean isStandalone) {
    this.isStandalone = isStandalone;
    this.setLayout(new BorderLayout());

    Panel uppanel = new Panel();
    this.add(uppanel, BorderLayout.NORTH);
    
    uppanel.add( new Label("Background") );
    
    Checkbox icheck = new Checkbox("Image", true);
    icheck.addItemListener( new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED) {
          background.setImage(loader.getImage());
        } else if (e.getStateChange() == ItemEvent.DESELECTED) {
          background.setImage(null);
        }
      }
    });
    uppanel.add(icheck);
    
    TuplePanel bgpanel = new TuplePanel(bgcolor);
    bgpanel.addTupleEventListener( new TupleEventListener() {
      public void tupleStateChanged(TupleEvent e) {
	bgcolor = e.getColor3f();
	background.setColor(bgcolor);
      }
    });
    uppanel.add(bgpanel);

    Panel downpanel = new Panel();
    downpanel.setLayout( new GridLayout(3, 1) );
    this.add(downpanel, BorderLayout.SOUTH);
    
    Panel[] dpanels = new Panel[3];
    for (int i=0; i<3; i++) {
      dpanels[i] = new Panel();
      downpanel.add(dpanels[i]);
    }
    
    dpanels[0].add( new Label("Text") );
    final TextField txfield = new TextField("", 24);
    dpanels[0].add(txfield);

    final Choice fchoice = new Choice();
    Font[] fonts
      = GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts();
    for (int i=0; i<fonts.length; i++) fchoice.add(fonts[i].getFontName());
    dpanels[0].add(fchoice);    

    dpanels[1].add(fcpanel);

    dpanels[1].add( new Label("Size") );
    final TextField szfield = new TextField("24");
    szfield.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        int size = 1;
        try {
          size = Integer.parseInt(e.getActionCommand());
          if (size < 1) size = 1;
          szfield.setText( Integer.toString(size) );
        } catch (NumberFormatException ex) {}
      }
    });
    dpanels[1].add(szfield);

    final Choice schoice = new Choice();
    schoice.add("PLAIN");
    schoice.add("BOLD");
    schoice.add("ITALIC");
    schoice.add("BOLD_ITALIC");
    dpanels[1].add(schoice);

    dpanels[2].add(fppanel);
    
    Button addbutton = new Button("Add");
    addbutton.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        int size = 12;
        int style = java.awt.Font.PLAIN;

        try {
          size = Integer.parseInt(szfield.getText());
          if (size < 1) size = 1;
        } catch (NumberFormatException ex) {}
        
        if (schoice.getSelectedItem().equals("PLAIN")) {
          style = java.awt.Font.PLAIN;
        } else if (schoice.getSelectedItem().equals("BOLD")) {
          style = java.awt.Font.BOLD;
        } else if (schoice.getSelectedItem().equals("ITALIC")) {
          style = java.awt.Font.ITALIC;
        } else if (schoice.getSelectedItem().equals("BOLD_ITALIC")) {
          style = java.awt.Font.BOLD | java.awt.Font.ITALIC;
        }
        
        Transform3D t3d = new Transform3D();
        t3d.setTranslation( new Vector3f(fppanel.getTuple3f()) );
        TransformGroup trans = new TransformGroup(t3d);
        trans.addChild(
          new Text2D( txfield.getText(),
                      new Color3f(fcpanel.getTuple3f()),
                      fchoice.getSelectedItem(),
                      size, style ) );
        universe.getLocale().removeBranchGraph(scene);
        scene.addChild(trans);
        universe.addBranchGraph(scene);
      }
    });
    dpanels[2].add(addbutton);

  }
  
  public void init() {}

  public void start() {
    GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();
    this.canvas = new Canvas3D(config);
    this.add(this.canvas, BorderLayout.CENTER);
    
    universe = new SimpleUniverse(canvas);
    universe.getViewingPlatform().setNominalViewingTransform();
    
    scene = createSceneGraph();
    universe.addBranchGraph(scene);
  }
  
  private BranchGroup createSceneGraph() {
    BranchGroup root = new BranchGroup();
    root.setCapability(BranchGroup.ALLOW_DETACH);
    
    BoundingSphere bounds = new BoundingSphere(new Point3d(), 10000.0);

    background = createBackground(bounds);
    root.addChild(background);

    return root;
  }
  
  private Background createBackground(Bounds bounds) {
    // Texture ̐
    Image image = null;
    if (this.isStandalone) {
      // AvP[VƂĎsĂ
      Toolkit toolkit = Toolkit.getDefaultToolkit();
      image = toolkit.getImage("bg.jpg");
    } else {
      // AvbgƂĎsĂ
      image = getImage(getCodeBase(), "bg.jpg");
    }
    MediaTracker mt = new MediaTracker(this);
    mt.addImage(image, 0);
    mt.checkAll(true);
    try { mt.waitForID(0); } catch (InterruptedException e) { e.printStackTrace(); }
    this.loader = new TextureLoader(image, this);
    Background bg = new Background(this.loader.getImage());
    bg.setCapability(Background.ALLOW_COLOR_READ);
    bg.setCapability(Background.ALLOW_COLOR_WRITE);
    bg.setCapability(Background.ALLOW_IMAGE_WRITE);
    bg.setApplicationBounds(bounds);
    return bg;
  }
  
  public static void main(String[] args) {
    Text2DTest applet = new Text2DTest();
    Frame frame = new MainFrame(applet, 500, 500);
  }
}
