// Java 3D Test Applet
// Text3DTest.java
//   Copyright (c) 1999 ENDO Yasuyuki
//                      mailto:yasuyuki@javaopen.org
//                      http://www.javaopen.org/j3dbook/index.html

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.universe.SimpleUniverse;

public class Text3DTest extends Applet {
  private Canvas3D canvas = null;
  private SimpleUniverse universe = null;
  private BranchGroup scene = null;
  private TransformGroup trans = null;
  private TransformGroup strans = null;
  private float scale = 0.2f;
  private Material mat = null;
  private Font[] fonts = null;
  private int nfont = 0;
  private Font font = null;
  private int style = java.awt.Font.PLAIN;
  private String text = "Java 3D";
  private Text3D text3d = null;

  private Color3f fontcolor = new Color3f(1.0f, 0.0f, 0.0f); // եȿ 
  private TuplePanel fcpanel = new TuplePanel(fontcolor);

  private Point3f fontposition = new Point3f(0.0f, 0.0f, 0.0f); // եȰ 
  private TuplePanel fppanel = new TuplePanel(fontposition);

  public Text3DTest() {
    this.setLayout(new BorderLayout());

    Panel downpanel = new Panel();
    downpanel.setLayout( new GridLayout(3, 1) );
    this.add(downpanel, BorderLayout.SOUTH);
    
    Panel[] dpanels = new Panel[3];
    for (int i=0; i<3; i++) {
      dpanels[i] = new Panel();
      dpanels[i].setLayout( new FlowLayout(FlowLayout.LEFT) );
      downpanel.add(dpanels[i]);
    }
    
    dpanels[0].add( new Label("Text") );
    final TextField txfield = new TextField(text, 24);
    txfield.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        text = e.getActionCommand();
        universe.getLocale().removeBranchGraph(scene);
        text3d.setString(text);
        universe.addBranchGraph(scene);
      }
    });
    dpanels[0].add(txfield);

    final Choice fchoice = new Choice();
    fonts =
      GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts();
    font = fonts[0];
    System.out.println("size=" + font.getSize());
    for (int i=0; i<fonts.length; i++) fchoice.add(fonts[i].getFontName());
    fchoice.addItemListener( new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        nfont = fchoice.getSelectedIndex();
        font = fonts[nfont].deriveFont(style);

        Font3D f3d = new Font3D( font, new FontExtrusion() );
	universe.getLocale().removeBranchGraph(scene);
        text3d.setFont3D(f3d);
        universe.addBranchGraph(scene);
      }
    });
    dpanels[0].add(fchoice);

    fcpanel.addTupleEventListener( new TupleEventListener() {
      public void tupleStateChanged(TupleEvent e) {
	fontcolor = e.getColor3f();
	mat.setDiffuseColor(fontcolor);
      }
    });
    dpanels[1].add(fcpanel);

    dpanels[1].add( new Label("Scale") );
    final TextField szfield = new TextField(Float.toString(scale));
    szfield.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
          scale = Float.parseFloat(e.getActionCommand());
          Transform3D t3d = new Transform3D();
          t3d.setScale(scale);
          strans.setTransform(t3d);
        } catch (NumberFormatException ex) {}
      }
    });
    dpanels[1].add(szfield);

    final Choice schoice = new Choice();
    schoice.add("PLAIN");
    schoice.add("BOLD");
    schoice.add("ITALIC");
    schoice.add("BOLD_ITALIC");
    schoice.addItemListener( new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String item = (String)e.getItem();
        if (item.equals("PLAIN")) {
          style = java.awt.Font.PLAIN;
        } else if (item.equals("BOLD")) {
          style = java.awt.Font.BOLD;
        } else if (item.equals("ITALIC")) {
          style = java.awt.Font.ITALIC;
        } else if (item.equals("BOLD_ITALIC")) {
          style = java.awt.Font.BOLD | java.awt.Font.ITALIC;
        }
        font = fonts[nfont].deriveFont(style);
        Font3D f3d = new Font3D( font, new FontExtrusion() );
        
        universe.getLocale().removeBranchGraph(scene);
        text3d.setFont3D(f3d);
        universe.addBranchGraph(scene);
      }
    });
    dpanels[1].add(schoice);

    fppanel.addTupleEventListener( new TupleEventListener() {
      public void tupleStateChanged(TupleEvent e) {
	Transform3D t3d = new Transform3D();
	trans.getTransform(t3d);
	t3d.setTranslation( new Vector3f(e.getTuple3f()) );
	trans.setTransform(t3d);
      }
    });
    dpanels[2].add(fppanel);
    
    Choice achoice = new Choice();
    achoice.add("ALIGN_FIRST");
    achoice.add("ALIGN_CENTER");
    achoice.add("ALIGN_LAST");
    achoice.addItemListener( new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String item = (String)e.getItem();
        int align = text3d.getAlignment();
        if (item.equals("ALIGN_FIRST")) {
          align = Text3D.ALIGN_FIRST;
        } else if (item.equals("ALIGN_CENTER")) {
          align = Text3D.ALIGN_CENTER;
        } else if (item.equals("ALIGN_LAST")) {
          align = Text3D.ALIGN_LAST;
        }
        universe.getLocale().removeBranchGraph(scene);
        text3d.setAlignment(align);
        universe.addBranchGraph(scene);
      }
    });
    dpanels[2].add(achoice);
    
    Choice pchoice = new Choice();
    pchoice.add("PATH_DOWN");
    pchoice.add("PATH_LEFT");
    pchoice.add("PATH_RIGHT");
    pchoice.add("PATH_UP");
    pchoice.select(2);
    pchoice.addItemListener( new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String item = (String)e.getItem();
        int path = text3d.getPath();
        if (item.equals("PATH_DOWN")) {
          path = Text3D.PATH_DOWN;
        } else if (item.equals("PATH_LEFT")) {
          path = Text3D.PATH_LEFT;
        } else if (item.equals("PATH_RIGHT")) {
          path = Text3D.PATH_RIGHT;
        } else if (item.equals("PATH_UP")) {
          path = Text3D.PATH_UP;
        }
        universe.getLocale().removeBranchGraph(scene);
        text3d.setPath(path);
        universe.addBranchGraph(scene);
      }
    });
    dpanels[2].add(pchoice);

    GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();
    canvas = new Canvas3D(config);
    this.add(canvas, BorderLayout.CENTER);
    
    universe = new SimpleUniverse(canvas);
    universe.getViewingPlatform().setNominalViewingTransform();
    
    scene = createSceneGraph();
    universe.addBranchGraph(scene);
  }
  
  private BranchGroup createSceneGraph() {
    BranchGroup root = new BranchGroup();
    root.setCapability(BranchGroup.ALLOW_DETACH);
    
    BoundingSphere bounds = new BoundingSphere(new Point3d(), 10000.0);

    root.addChild( createLight(0.3f, -0.3f, -0.3f, bounds) );
    
    AmbientLight alight = new AmbientLight();
    alight.setInfluencingBounds(bounds);
    root.addChild(alight);
    
    trans = new TransformGroup();
    trans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    trans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    root.addChild(trans);
    
    Transform3D t3d = new Transform3D();
    t3d.setScale(scale);
    
    strans = new TransformGroup(t3d);
    strans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    strans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    trans.addChild(strans);

    text3d =
      new Text3D( new Font3D( font, new FontExtrusion() ),
                  text,
                  fontposition,
                  Text3D.ALIGN_FIRST,
                  Text3D.PATH_RIGHT );
    text3d.setCapability(Text3D.ALLOW_FONT3D_WRITE);
    text3d.setCapability(Text3D.ALLOW_STRING_WRITE);
    text3d.setCapability(Text3D.ALLOW_ALIGNMENT_READ);
    text3d.setCapability(Text3D.ALLOW_ALIGNMENT_WRITE);
    text3d.setCapability(Text3D.ALLOW_PATH_READ);
    text3d.setCapability(Text3D.ALLOW_PATH_WRITE);
    
    Shape3D shape = new Shape3D(text3d, createAppearance());
   
    strans.addChild(shape);

    return root;
  }
  
  private Light createLight(float x, float y, float z, Bounds bounds) {
    DirectionalLight light =
      new DirectionalLight( true,
                            new Color3f(1.0f, 1.0f, 1.0f),
                            new Vector3f(x, y, z));
    light.setInfluencingBounds(bounds);
    
    return light;
  }

  private Appearance createAppearance() {
    Appearance app = new Appearance();
    mat = new Material();
    mat.setCapability(Material.ALLOW_COMPONENT_READ);
    mat.setCapability(Material.ALLOW_COMPONENT_WRITE);
    mat.setDiffuseColor(fontcolor);
    mat.setShininess(100.0f);
    app.setMaterial(mat);
    return app;
  }

  public static void main(String[] args) {
    Text3DTest applet = new Text3DTest();
    Frame frame = new MainFrame(applet, 500, 500);
  }
}
