// Java 2D Test Program
// ShapePanel.java
//   Copyright (c) 1999 ENDO Yasuyuki
//                      mailto:yasuyuki@javaopen.org
//                      http://www.javaopen.org/j3dbook/index.html

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.util.*;

public class ShapePanel extends Panel {
  private double factor = 100.0;
  private Shape shape = null;
  
  public ShapePanel() {
    //shape =
    //  new Line2D.Double(0.0, 0.0, 1.0, 0.0);
    //shape =
    //  new CubicCurve2D.Double(0.0, 0.0, 0.2, 0.7, 0.7, 0.8 1.0, 0.0);
    shape =
      new QuadCurve2D.Double(0.0, 0.0, 1.0, 0.0, 1.0, 1.0);
  }
  
  public ShapePanel(Shape aShape) {
    shape = aShape;
  }
  
  public ShapePanel(Shape aShape, double aFactor) {
    shape = aShape;
    factor = aFactor;
  }
  
  public void setShape(Shape aShape) {
    shape = aShape;
    //Graphics g = this.getGraphics();
    //this.paint(g);
    this.repaint();
  }
  public Shape getShape() { return shape; }
  
  public void paint(Graphics g) {
    Graphics2D g2d = (Graphics2D)g;

    Rectangle rect = this.getBounds();

    g2d.setPaint(java.awt.Color.red);
    g2d.setStroke(
      new BasicStroke( 1.0f / (float)factor * 10.0f,
                       BasicStroke.CAP_ROUND,
                       BasicStroke.JOIN_MITER ) );
    g2d.scale(factor, factor);
    if (shape != null) g2d.draw(shape);
    
    int nx = rect.width / (int)factor;
    int ny = rect.height / (int)factor;

    g2d.setPaint(java.awt.Color.green);
    g2d.setStroke( new BasicStroke(1.0f / (float)factor) );
    double x2 = (double)rect.width / factor;
    double y2 = (double)rect.height / factor;
    //System.out.println("x2=" + x2 + ", y2=" + y2);
    
    for (int i=0; i<nx; i++) {
      double xs = (double)(i + 1);
      g2d.draw( new Line2D.Double(xs, 0.0, xs, y2) );
      for (int j=0; j<ny; j++) {
        double y = (double)(j + 1);
        g2d.draw( new Line2D.Double(0.0, y, x2, y) );
      }
    }
  }
  
  public Dimension getPreferredSize() {
    int w = 100;
    int h = 100;
    Rectangle2D rect = shape.getBounds2D();
    int rw = (int)(rect.getWidth() * factor);
    int rh = (int)(rect.getHeight() * factor);
    if (rw > w) w = rw;
    if (rh > h) h = rh;
    //System.out.println("w=" + w + ", h=" + h);//DEBUG
    return new Dimension(w, h);
  }

  public static void main(String[] args) {
    ShapePanel me = new ShapePanel();
    Frame f = new Frame();
    f.setLayout( new BorderLayout() );
    f.add(me, BorderLayout.CENTER);
    f.pack();
    f.show();
  }
  
}
