
import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.GraphicsConfiguration;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.MediaTracker;
import javax.vecmath.Point3d;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.Appearance;
import javax.media.j3d.Texture;
import javax.media.j3d.Alpha;
import javax.media.j3d.RotationInterpolator;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.behaviors.mouse.MouseRotate;
import com.sun.j3d.utils.behaviors.mouse.MouseTranslate;
import com.sun.j3d.utils.behaviors.mouse.MouseZoom;
import com.sun.j3d.utils.image.TextureLoader;

public class EarthApplet extends Applet {
  private boolean standalone;
  private boolean isStandalone() { return standalone; }
  private void setStandalone(boolean standaloneState) {
    standalone = standaloneState;
  }

  public EarthApplet() {
     this(false);
  }

  public EarthApplet(boolean standaloneState) {
    setStandalone(standaloneState);

    GraphicsConfiguration config =
      SimpleUniverse.getPreferredConfiguration();
    Canvas3D canvas = new Canvas3D(config);
    this.setLayout( new BorderLayout() );
    this.add(canvas, BorderLayout.CENTER);

    BranchGroup scene = createSceneGraph();

    SimpleUniverse universe = new SimpleUniverse(canvas);
    universe.getViewingPlatform().setNominalViewingTransform();

    universe.addBranchGraph(scene);
  }

  private BranchGroup createSceneGraph() {
    BranchGroup root = new BranchGroup();

    TransformGroup trans = new TransformGroup();
    trans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    trans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    root.addChild(trans);

    BoundingSphere bounds = new BoundingSphere(new Point3d(), 100.0);

    MouseRotate mouseRotate = new MouseRotate(trans);
    mouseRotate.setSchedulingBounds(bounds);
    root.addChild(mouseRotate);

    MouseTranslate mouseTranslate = new MouseTranslate(trans);
    mouseTranslate.setSchedulingBounds(bounds);
    root.addChild(mouseTranslate);

    MouseZoom mouseZoom = new MouseZoom(trans);
    mouseZoom.setSchedulingBounds(bounds);
    root.addChild(mouseZoom);

    TransformGroup rotationTrans = new TransformGroup();
    rotationTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    rotationTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    trans.addChild(rotationTrans);

    Alpha alpha = new Alpha(-1, 4000);
    RotationInterpolator rotationInterpolator =
      new RotationInterpolator(alpha, rotationTrans);
    rotationInterpolator.setSchedulingBounds(bounds);
    root.addChild(rotationInterpolator);

    Sphere earth = new Sphere( 0.4f,
                               Sphere.GENERATE_TEXTURE_COORDS,
                               createAppearance() );

    rotationTrans.addChild(earth);

    return root;
  }

  private Appearance createAppearance() {
    Appearance app = new Appearance();

    Image image;
    if (this.isStandalone()) {
       final Toolkit toolkit = Toolkit.getDefaultToolkit();
       image = toolkit.getImage("earth.jpg");
    } else {
       image = getImage(getCodeBase(), "earth.jpg");
    }
    MediaTracker mt = new MediaTracker(this);
    mt.addImage(image, 0);
    mt.checkAll(true);
    try { mt.waitForID(0); } catch (InterruptedException e) { e.printStackTrace(); }

    Texture texture = new TextureLoader(image, this).getTexture();

    app.setTexture(texture);

    return app;
  }

  public static void main(String[] args) {
    EarthApplet applet = new EarthApplet(true);
    MainFrame frame = new MainFrame(applet, 500, 500);
  }

}

